#include "todolist.h"
#include <QDomDocument>
#include <QDomElement>
#include <QTextStream>

/**
 * Singleton
 */
ToDoList *ToDoList::_instance = NULL;

/**
 * Constructeur privé
 */
ToDoList::ToDoList()
{
}

/**
 * Destructeur
 */
ToDoList::~ToDoList()
{
	delete(_instance);
	_instance = NULL;
}


/**
 * Methode statique permettant la creation et la récuperation du singleton
 * @return instance unique
 */
ToDoList* ToDoList::getInstance()
{
	if(_instance == NULL)
	{
			_instance = new ToDoList();
	}
	
	return _instance;
}

/**
 * Methode permettant de savoir la liste de tache contient une tache X
 * @param t objet tache à retrouver
 * @
 */
bool ToDoList::containsTask(Task *t)
{
		bool res = false;
		
		unsigned int i;
		for(i=0; i < this->tasks.size(); i++)
		{
			if(this->tasks.at(i)->equals(t))
			{
				res = true;
				break;
			}
		}
		
		return res;
}

/**
 * Methode permettant de supprimer une tache de la liste
 * @param nom de la tache à supprimer
 */
bool ToDoList::removeTask(QString taskName)
{
	bool found = false;
	
		unsigned int i;
		for(i=0; i < this->tasks.size(); i++)
		{
			if(this->tasks.at(i)->getName() == taskName)
			{
				found = true;
				break;
			}
		}
		
		if(found)
			this->tasks.erase(this->tasks.begin() + i);
			
	return found;
}

/**
 * Methode permettant d'ajouter une tache en fin de liste
 * @param instance à ajouter
 */
bool ToDoList::addTask(Task *t)
{
	bool res = false;
	if(!this->containsTask(t))
	{
		this->tasks.push_back(t);
		res = true;
	}
	
	return res;
}

/**
 * Methode permettant de rechercher une tache en fonction de son nom
 * @param nom de la tache recherché
 */
Task* ToDoList::getTask(QString name)
{
	Task* res = NULL;
	
	unsigned int i;
	for(i=0; i < this->tasks.size(); i++)
	{
		if(this->tasks.at(i)->getName().toLower() == name.toLower())
		{
			res = this->tasks.at(i);
		}
	}
	
	return res;

}

/**
 * Retourne toute la liste des taches
 */
std::vector<Task*> ToDoList::getTasks()
{
	return  this->tasks;
}

/**
 * Retourne une sous-liste des taches en cours (non-terminées)
 */
std::vector<Task*> ToDoList::getCurrentTasks()
{
	std::vector<Task*> res;
	
	unsigned int i;
	for(i=0; i < this->tasks.size(); i++)
	{
		if(!this->tasks.at(i)->isFinished())
		{
			res.push_back(this->tasks.at(i));
		}
	}
	
	return res;
}

/**
 * Retourne la liste des taches terminées
 */
std::vector<Task*> ToDoList::getFinishedTasks()
{
	std::vector<Task*> res;
	
	unsigned int i;
	for(i=0; i < this->tasks.size(); i++)
	{
		if(this->tasks.at(i)->isFinished())
		{
			res.push_back(this->tasks.at(i));
		}
	}
	
	return res;

}

/**
 * Retourne la liste de tags
 */
std::vector<QString> ToDoList::getTagsList()
{
	return this->tags;
}

/**
 * Methode permettant d'ajouter un tags à la liste
 * @param nom du tag
 */
bool ToDoList::addTag(QString newtag)
{
	bool res = this->containsTag(newtag);
	if(!res)
		this->tags.push_back(newtag);
		
	return res;
}

/**
 * Permet de verifier si le tag est deja contenue dans la liste
 */
bool ToDoList::containsTag(QString tag)
{
	bool res = false;
	unsigned int i;
	for(i=0; i< this->tags.size(); i++)
	{
		if(this->tags.at(i) == tag)
		{
			res = true;
			break;
		}
	}
	
	return res;
}

/**
 * Supprime un tag de la liste
 */
bool ToDoList::removeTag(QString tagName)
{
	bool found = false;

		unsigned int i;
		for(i=0; i < this->tags.size(); i++)
		{
			if(this->tags.at(i) == tagName)
			{
				found = true;
				break;
			}
		}
		
		if(found)
		{
			//suppression dans la liste
			this->tags.erase(this->tags.begin() + i);
			
			//supression dans les taches
			unsigned int j;
			for(j=0; j < this->tasks.size(); j++)
			{
				if(this->tasks.at(j)->containsTag(tagName))
				{
						this->tasks.at(j)->removeTag(tagName);
				}
			}
		}
			
	return found;
}

/**
 * Fonction retournant le nombre de taches actives
 */
int ToDoList::countCurrentTasks()
{
		return (int) this->getCurrentTasks().size();
}

/**
 * Fonction retournant le nombre de taches finies
 */
int ToDoList::countFinishedTasks()
{
		return (int) this->getFinishedTasks().size();
}

/**
 * Retourne le nombre de taches totales
 */
int ToDoList::countTasks()
{
		return (int) this->tasks.size();
}

/**
 * Permet de transformer la liste en chaine de caractere pour l'affichage sur l'entrée standard
 */
QString ToDoList::toString()
{
	QString res;
	
		unsigned int i;
		for(i = 0; i < this->tasks.size(); i++)
		{
				res += this->tasks.at(i)->toString();
				res += "\n";
		}
		
	return res;
}

/**
 * Permet de parser le fichier XML (taches.xml) en le transformant en structure de taches et de tags
 * XML FORMAT :
 * <todolist>
 * <tags>
 *		<tag>NOM DE L’ETIQUETTE</tag>
		{...}
    <tags>
	<task>
       <name>NOM DE LA TACHE</name>
       <priority>PRIORITE DE LA TACHE</priority>
       <status>ETAT DE LA TACHE</status>
       <description>DESCRIPTION DE LA TACHE</description>
       <deadline>ECHEANCE</deadline>
       <create>DATE DE CREATION</create>
       <tags>
           <tag>NOM DE L’ETIQUETTE</tag>
           {...}
       </tags>
   </task>
  {...}
</todolist>
 */
bool ToDoList::openXML()
{
	//on vide la liste des taches
	this->tasks.clear();
				
	//on creer le domDocument
	QDomDocument doc;
			
	//on ouvre le fichier XML
	QFile file("./saves/taches.xml");
	
	if (!file.open(QIODevice::ReadOnly))
		return FALSE;
			
	if (!doc.setContent(&file))
	{
		file.close();
		return FALSE;
	}
	
	//Lecture de la racine du fichier XML ("todolist")
	QDomElement root=doc.documentElement();
	
	//Lecture d'un fils de "todolist"
	QDomElement child=root.firstChild().toElement();
	while(!child.isNull())
	{
	    //Lecture d'une nouvelle tache
	    if (child.tagName() == "task")
	    {
		  //On cree une nouvelle tache
	      Task *t = new Task();
	      
	      //On parcours la "task"
	      QDomElement grandChild=child.firstChild().toElement();
	      while(!grandChild.isNull())
	      {
				//On construit la bouvelle tache
				if (grandChild.tagName() == "name")
				{
					//Ajout du nom
					t->setName(grandChild.text());
				}
				else if (grandChild.tagName() == "priority")
				{
					//Ajout de la priorite
					if(!grandChild.text().isEmpty())
						t->setPriority(grandChild.text().toInt());
				}
				else if (grandChild.tagName() == "description")
				{
					//Ajout de la description
					t->setDescription(grandChild.text());
				}
				else if (grandChild.tagName() == "status")
				{
					//Ajout du status
					if(!grandChild.text().isEmpty() && grandChild.text().compare("1") == 0)
						t->finish();
				}
				else if (grandChild.tagName() == "deadline")
				{
					//Ajout de l'echeance
					t->setDeadLine(QDate::fromString(grandChild.text(),"dd.MM.yyyy"));
				}
				else if (grandChild.tagName() == "create")
				{
					//Ajout de la date de creation
					t->setCreateDate(QDate::fromString(grandChild.text(),"dd.MM.yyyy"));
				}
				else if(grandChild.tagName() == "tags")
				{
					//Parcours de la liste de tags
					QDomElement tagChild = grandChild.firstChild().toElement();
					while(!tagChild.isNull())
					{
							if(tagChild.tagName() == "tag" && !tagChild.text().isEmpty())
							{
								//Ajout du tag
								t->addTag(tagChild.text());
							}
							
						//suivant
						tagChild = tagChild.nextSibling().toElement();
					}
				}
				
				grandChild = grandChild.nextSibling().toElement();
		   } 
		   
		   //On ajoute la nouvelle tache dans la liste
			this->addTask(t);
	    }
	    else if(child.tagName() == "tags")
	    {
		   //On parcours la liste de "tags"
	      QDomElement grandChild=child.firstChild().toElement();
	      while(!grandChild.isNull())
	      {
				if (grandChild.tagName() == "tag")
				{
					//Ajout du tag
					this->addTag(grandChild.text());
				}
			grandChild = grandChild.nextSibling().toElement();
		  }
		}
	   
		//On continu de parcours l'arbre (nouvelle tache ou taglist)
	    child = child.nextSibling().toElement();
	  }
	  
	return TRUE;
}


/**
 * Transforme la todoList en un fichier XML (enregistrement des données)
 */
bool ToDoList::saveXML()
{
	//on creer le nouveau document XML
	QDomDocument doc;
	
	//on cree la racine
	QDate date = QDate::currentDate();
	QDomElement root = doc.createElement("todolist");
	root.setAttribute("author", "autogenerated by ToDoList Manager");
	root.setAttribute("revision", date.toString("dd.MM.yyyy"));
	
	//on ajoute la racine
	doc.appendChild(root);
	
	//on parcours la liste des taches
	unsigned int i;
	for(i=0; i < this->tasks.size(); i++)
	{
		//creation de la tache
		QDomElement curtask = doc.createElement("task");
		
		//ajout du name
		QDomElement name = doc.createElement("name");
		name.appendChild(doc.createTextNode(this->tasks.at(i)->getName()));
		curtask.appendChild(name);
		
		//ajout de la description
		QDomElement desc = doc.createElement("description");
		desc.appendChild(doc.createTextNode(this->tasks.at(i)->getDescription()));
		curtask.appendChild(desc);
		
		//ajout de la priorite
		QDomElement priority = doc.createElement("priority");
		QString str_p;
		str_p.setNum(this->tasks.at(i)->getPriority());
		priority.appendChild(doc.createTextNode(str_p));
		curtask.appendChild(priority);
		
		//ajout du status
		QDomElement status = doc.createElement("status");
		QString str_status;
		str_status.setNum((int) this->tasks.at(i)->isFinished());
		status.appendChild(doc.createTextNode(str_status));
		curtask.appendChild(status);
		
		//ajout de la deadline
		QDomElement deadline = doc.createElement("deadline");
		deadline.appendChild(doc.createTextNode(this->tasks.at(i)->getDeadLine().toString("dd.MM.yyyy")));
		curtask.appendChild(deadline);
		
		//ajout de la date de creation
		QDomElement create = doc.createElement("create");
		create.appendChild(doc.createTextNode(this->tasks.at(i)->getCreateDate().toString("dd.MM.yyyy")));
		curtask.appendChild(create);
		
		//ajout de la liste des tags
		QDomElement tags = doc.createElement("tags");
		unsigned int y;
		for(y=0; y < this->tasks.at(i)->getTags().size();y++)
		{
			//ajout d'un nouveau tag
			QDomElement curtag = doc.createElement("tag");
			curtag.appendChild(doc.createTextNode(this->tasks.at(i)->getTags().at(y)));
			tags.appendChild(curtag);
		}
		curtask.appendChild(tags);
		
		//ajout de la tache dans la racine
		root.appendChild(curtask);
		
	 }
	 
	 //creation de la liste des tags
	QDomElement listtags = doc.createElement("tags");
	
	 //on parcours la liste des tags
	for(i = 0; i < this->getTagsList().size();i++)
	{
		//ajout d'un nouveau tag dans la liste
		QDomElement curtag = doc.createElement("tag");
		curtag.appendChild(doc.createTextNode(this->tags.at(i)));
		listtags.appendChild(curtag);	
	}
	
	//ajout dans la liste dans la racine
	root.appendChild(listtags);

	// Création d'un objet QFile
	QFile file("./saves/taches.xml");
		
	// On ouvre notre fichier en lecture seule et on verifie l'ouverture
	if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
		return FALSE;
			
	QTextStream flux(&file);
	flux.setCodec("UTF-8");
	flux << doc.toString() << endl;
		
	file.close();

	return TRUE;
}
