#include "createtaskdialog.h"
#include "task.h"
#include "todolist.h"

/**
 * Constructeur 
 */
CreateTaskDialog::CreateTaskDialog(QWidget *parent)
 : QDialog(parent)
{
		//construction et initialisation
		this->setupUi();
		this->initUi();
		
		//connexion des evenements
		QObject::connect(this->cancelButton, SIGNAL(clicked()), this, SLOT(close()));
		QObject::connect(this->okButton, SIGNAL(clicked()), this, SLOT(createTask()));
}

/**
 * Fonction permettant la contruction de la boite de dialogue
 */
void CreateTaskDialog::setupUi()
{
		//dialog options
		this->resize(400,300);
		this->setWindowTitle(QString::fromUtf8("Création d'une tâche"));
		this->setModal(true);
		
		//layout et containers
		QWidget *informationTab = new QWidget();
		QWidget *tagsTab = new QWidget();
		QVBoxLayout *v_main_layout = new QVBoxLayout();
		QGridLayout *grid_buttons_layout = new QGridLayout();
		QFormLayout *form_infos_layout = new QFormLayout();	
		QVBoxLayout *v_tags_layout = new QVBoxLayout();
		
		//creation du widget à onglet
		this->editTabWidget = new QTabWidget(this);
		
		//creation des deux boutons
		this->cancelButton = new QPushButton("Annuler");
		this->okButton = new QPushButton(QString::fromUtf8("Créer"));
		
		//creation des widgets du premier onglet
		this->nameField = new QLineEdit();
		this->nameField->setToolTip(QString::fromUtf8("Le nom est obligatoire et unique"));
		this->priorityCombo = new QComboBox();
		this->priorityCombo->addItem("Inconnue");
		this->priorityCombo->addItem("Haute");
		this->priorityCombo->addItem("Moyenne");
		this->priorityCombo->addItem("Faible");
		this->descEditor = new QTextEdit();
		this->descEditor->setToolTip(QString::fromUtf8("Tapez la description de votre tâche"));
		this->deadlineCalendar = new QCalendarWidget();
		this->deadlineCalendar->setToolTip(QString::fromUtf8("Choisir l'échéance de la tâche. Par défaut, c'est la date du jour"));
		//creation des widgets du second onglet
		QLabel *tagsLabel = new QLabel(QString::fromUtf8("Choisir les étiquettes de la tâche :"));
		this->tagsListWidget = new QListWidget();
		this->tagsListWidget->setToolTip(QString::fromUtf8("Cliquez sur l'étiquette pour l'affecter à la tâche"));
		this->tagsListWidget->setSelectionMode(QAbstractItemView::MultiSelection);
		this->tagsListWidget->setSortingEnabled(true);
		
		//alignement des boutons
		grid_buttons_layout->addWidget(new QWidget(),0,0,1,2);
		grid_buttons_layout->addWidget(cancelButton,0,2);
		grid_buttons_layout->addWidget(okButton,0,3);
		
		//alignement des widget du premier onglet
		informationTab->setLayout(form_infos_layout);
		form_infos_layout->setHorizontalSpacing(30);
		form_infos_layout->addRow("Nom :",nameField);
		form_infos_layout->addRow("Description :",descEditor);
		form_infos_layout->addRow(QString::fromUtf8("Priorité :"), priorityCombo);
		form_infos_layout->addRow(QString::fromUtf8("Echéance :"), deadlineCalendar);
		
		//alignement des widget du second onglet
		tagsTab->setLayout(v_tags_layout);
		v_tags_layout->addWidget(tagsLabel);
		v_tags_layout->addWidget(this->tagsListWidget);
		
		//ajout des deux onglets
		editTabWidget->addTab(informationTab, "Informations");
		editTabWidget->addTab(tagsTab, QString::fromUtf8("Étiquettes"));
		
		v_main_layout->addWidget(editTabWidget);
		v_main_layout->addLayout(grid_buttons_layout);
		this->setLayout(v_main_layout);
}

/**
 * Methode permettant d'initialiser les composants de la boite de dialogue
 */
void CreateTaskDialog::initUi()
{

	//remplissage par defaut des widgets
	this->nameField->setText("");
	this->descEditor->setText("");
	this->priorityCombo->setCurrentIndex(0);
	this->deadlineCalendar->setSelectedDate(QDate::currentDate());
		
	//remplissage de la liste des tags
	std::vector<QString> tags = ToDoList::getInstance()->getTagsList();
	unsigned int i;
	for(i=0;i < tags.size(); i++)
	{
			QListWidgetItem * item = new QListWidgetItem(tags.at(i));
			this->tagsListWidget->addItem(item);					
	}
	this->tagsListWidget->sortItems();
}

/**
 * Methode appelé lors du clic sur le bouton "OK" permettant de créer une nouvelle tache
 */
void CreateTaskDialog::createTask()
{
	if(this->nameField->text() == "")
	{
		QMessageBox::warning(this,"Attention",QString::fromUtf8("Le nom d'une tâche ne peut pas être vide !"));
	}
	else if(ToDoList::getInstance()->getTask(this->nameField->text()) != NULL)
	{
		QMessageBox::warning(this,"Attention",QString::fromUtf8("Le nom d'une tâche est deja utilisé !"));
	}
	else
	{
		//on cree la nouvelle tache
		Task* t = new Task(this->nameField->text());
		
		//on met a jour ses informations
		t->setDescription(this->descEditor->toPlainText());
		t->setPriority(this->priorityCombo->currentIndex());
		t->setDeadLine(this->deadlineCalendar->selectedDate());
			
		//on met à jours ses tags
		QList<QListWidgetItem*> tags = this->tagsListWidget->selectedItems();
		int i;
		for(i = 0; i < tags.size(); i++)
		{
				t->addTag(tags.at(i)->text());
		}
		
		//ajout de la nouvelle tache
		ToDoList::getInstance()->addTask(t);
		
		this->accept();
	}
}

/**
 * Destructeur
 */
CreateTaskDialog::~CreateTaskDialog() {}

