#include "edittaskdialog.h"
#include "task.h"
#include "todolist.h"

/**
 * Constructeur
 */
EditTaskDialog::EditTaskDialog(QWidget *parent, QString taskNameId)
 : QDialog(parent)
{
		this->taskNameId = taskNameId;
		this->setupUi();
		this->initUi();
		
		QObject::connect(this->cancelButton, SIGNAL(clicked()), this, SLOT(close()));
		QObject::connect(this->okButton, SIGNAL(clicked()), this, SLOT(editTask()));
}

/**
 * Methode permettant la construction de l'interface
 */
void EditTaskDialog::setupUi()
{
		//dialog options
		this->resize(400,300);
		this->setWindowTitle(QString::fromUtf8("Édition d'une tâche"));
		this->setModal(true);
		
		//layout et containers
		QWidget *informationTab = new QWidget();
		QWidget *tagsTab = new QWidget();
		QButtonGroup *group = new QButtonGroup();
		QVBoxLayout *v_main_layout = new QVBoxLayout();
		QGridLayout *grid_buttons_layout = new QGridLayout();
		QVBoxLayout *v_statut_layout = new QVBoxLayout();
		QFormLayout *form_infos_layout = new QFormLayout();	
		QVBoxLayout *v_tags_layout = new QVBoxLayout();
		
		//creation du widget à onglet
		this->editTabWidget = new QTabWidget(this);
		
		//creation des deux boutons
		this->cancelButton = new QPushButton("Annuler");
		this->okButton = new QPushButton("Modifier");
		
		//creation des widgets du premier onglet
		this->nameField = new QLineEdit();
		this->nameField->setToolTip(QString::fromUtf8("Le nom de la tâche doit être unique et non-vide"));
		this->priorityCombo = new QComboBox();
		this->priorityCombo->addItem("Inconnue");
		this->priorityCombo->addItem("Haute");
		this->priorityCombo->addItem("Moyenne");
		this->priorityCombo->addItem("Faible");
		this->descEditor = new QTextEdit();
		this->deadlineCalendar = new QCalendarWidget(),
		this->activeRButton = new QRadioButton(QString::fromUtf8("Tâche active"));
		this->finishedRButton = new QRadioButton(QString::fromUtf8("Tâche terminée"));
		
		//creation des widgets du second onglet
		QLabel *tagsLabel = new QLabel(QString::fromUtf8("Choisir les étiquettes de la tâche :"));
		this->tagsListWidget = new QListWidget();
		this->tagsListWidget->setToolTip(QString::fromUtf8("Cliquez sur l'étiquette pour l'affecter à la tâche"));
		this->tagsListWidget->setSelectionMode(QAbstractItemView::MultiSelection);
		this->tagsListWidget->setSortingEnabled(true);
		
		//alignement des statuts
		group->addButton(this->activeRButton);
		group->addButton(this->finishedRButton);
		v_statut_layout->addWidget(this->activeRButton);
		v_statut_layout->addWidget(this->finishedRButton);
		
		//alignement des boutons
		grid_buttons_layout->addWidget(new QWidget(),0,0,1,2);
		grid_buttons_layout->addWidget(cancelButton,0,2);
		grid_buttons_layout->addWidget(okButton,0,3);
		
		//alignement des widget du premier onglet
		informationTab->setLayout(form_infos_layout);
		form_infos_layout->setHorizontalSpacing(30);
		form_infos_layout->addRow("Nom :",nameField);
		form_infos_layout->addRow(QString::fromUtf8("Description :"),descEditor);
		form_infos_layout->addRow(QString::fromUtf8("Priorité :"), priorityCombo);
		form_infos_layout->addRow(QString::fromUtf8("Echéance :"), deadlineCalendar);
		form_infos_layout->addRow("Statut :",v_statut_layout);
		
		//alignement des widget du second onglet
		tagsTab->setLayout(v_tags_layout);
		v_tags_layout->addWidget(tagsLabel);
		v_tags_layout->addWidget(this->tagsListWidget);
		
		//ajout des deux onglets
		editTabWidget->addTab(informationTab, "Informations");
		editTabWidget->addTab(tagsTab, QString::fromUtf8("Étiquettes"));
		
		v_main_layout->addWidget(editTabWidget);
		v_main_layout->addLayout(grid_buttons_layout);
		this->setLayout(v_main_layout);
		
}

/**
 * Methode permettant l'initialisation par defaut des composants
 */
void EditTaskDialog::initUi()
{
		//on retrouve la tache à editer
		Task* te = ToDoList::getInstance()->getTask(this->getEditedTaskId());
		if(te != NULL)
		{
				//remplissage des widgets
				this->nameField->setText(te->getName());
				this->descEditor->setText(te->getDescription());
				this->priorityCombo->setCurrentIndex(te->getPriority());
				
				if(te->isFinished())
					this->finishedRButton->setChecked(true);
				else
					this->activeRButton->setChecked(true);
					
				this->deadlineCalendar->setSelectedDate(te->getDeadLine());
		
				//remplissage de la liste des tags
				std::vector<QString> tags = ToDoList::getInstance()->getTagsList();
				unsigned int i;
				for(i=0;i < tags.size(); i++)
				{
					QListWidgetItem * item = new QListWidgetItem(tags.at(i));
					this->tagsListWidget->addItem(item);

					if(te->containsTag(tags.at(i)))
					{
							item->setSelected(true);
					}
				}
				this->tagsListWidget->sortItems();
		}
		
		
}

/**
 * Methode permettant de retourner l'ancien nom de la tache en cours d'edition
 */
QString EditTaskDialog::getEditedTaskId()
{
		return this->taskNameId;
}

/**
 * Methode permettant de modifier le nom de la tache en cours d'édition
 */
void EditTaskDialog::setEditedTaskId(QString nameId)
{
		this->taskNameId = nameId;
}

/**
 * Slot permettant d'editer la tache
 */
void EditTaskDialog::editTask()
{
	if(this->nameField->text() == "")
	{
		QMessageBox::warning(this,"Attention",QString::fromUtf8("Le nom d'une tâche ne peut pas être vide !"));
	}
	else if(this->nameField->text() != this->getEditedTaskId() && ToDoList::getInstance()->getTask(this->nameField->text()) != NULL)
	{
		QMessageBox::warning(this,"Attention",QString::fromUtf8("Le nom d'une tâche est déjà utilisé !"));
	}
	else
	{
		//on retrouve la tache à editer
		Task* te = ToDoList::getInstance()->getTask(this->getEditedTaskId());
		
		//on met a jour ses informations
		te->setName(this->nameField->text());
		te->setDescription(this->descEditor->toPlainText());
		te->setPriority(this->priorityCombo->currentIndex());
		te->setDeadLine(this->deadlineCalendar->selectedDate());
		if(this->finishedRButton->isChecked())
			te->finish();
		else
			te->activate();
			
		//on met à jours ses tags
		te->clearTags();
		QList<QListWidgetItem*> tags = this->tagsListWidget->selectedItems();
		int i;
		for(i = 0; i < tags.size(); i++)
		{
				te->addTag(tags.at(i)->text());
		}
		
		this->accept();
	}
}

/**
 * Destructeur
 */
EditTaskDialog::~EditTaskDialog() {}
