#include "filtredialog.h"
#include "todolist.h"
#include "todoframe.h"

/**
 * Constructeur
 */
FiltreDialog::FiltreDialog(QWidget *parent) : QDialog(parent)
{
	this->setupUi();
	this->initUi();
	
	QObject::connect(this->okButton, SIGNAL(clicked()), this, SLOT(valid()));
	QObject::connect(this->cancelButton, SIGNAL(clicked()), this, SLOT(close()));
	QObject::connect(this->resetButton, SIGNAL(clicked()), this, SLOT(reset()));
}

/**
 * Destructeur
 */
FiltreDialog::~FiltreDialog() {}

/**
 * Construction de l'interface
 */
void FiltreDialog::setupUi()
{
	
		//dialog options
		this->resize(400,300);
		this->setWindowTitle(QString::fromUtf8("Création d'un filtre"));
		this->setModal(true);
		
		//creation des containers
		QVBoxLayout *v_main_layout = new QVBoxLayout();
		QGridLayout *grid_buttons_layout = new QGridLayout();
		
		//creation des widgets
		QLabel *label = new QLabel(QString::fromUtf8("Choisir les étiquettes du filtre :"));
		this->filtreListBox = new QListWidget();
		this->filtreListBox->setToolTip(QString::fromUtf8("Cliquez sur l'étiquette pour l'ajouter au filtre"));
		this->filtreListBox->setSelectionMode(QAbstractItemView::MultiSelection);
		this->filtreListBox->setSortingEnabled(true);
		this->okButton = new QPushButton("Appliquer");
		this->resetButton = new QPushButton(QString::fromUtf8("Réinitialiser"));
		this->cancelButton = new QPushButton("Annuler");
		
		//alignement des boutons
		grid_buttons_layout->addWidget(new QWidget(), 0,0,1,2);
		grid_buttons_layout->addWidget(this->resetButton, 0,2);
		grid_buttons_layout->addWidget(this->cancelButton, 0,3);
		grid_buttons_layout->addWidget(this->okButton, 0,4);
		
		//alignement global
		v_main_layout->addWidget(label);
		v_main_layout->addWidget(this->filtreListBox);
		v_main_layout->addLayout(grid_buttons_layout);
		this->setLayout(v_main_layout);
}


/**
 * Initialisation de l'interface
 */
void FiltreDialog::initUi()
{
	//remplissage de la liste des tags
	std::vector<QString> tags = ToDoList::getInstance()->getTagsList();
	unsigned int i;
	for(i=0;i < tags.size(); i++)
	{
			QListWidgetItem * item = new QListWidgetItem(tags.at(i));
			this->filtreListBox->addItem(item);
			
			//selection ?
			if(((ToDoFrame*) this->parent())->filtre.contains(tags.at(i)))
				item->setSelected(true);			
	}
	this->filtreListBox->sortItems();
}

/**
 * Slot permettant l'enregistrement des modification du filtre
 */
void FiltreDialog::valid()
{
	//RAZ de l'ancien filtre
	((ToDoFrame*) this->parent())->filtre.clear();
	
	//Creation du filtre
	QList<QListWidgetItem*> tags = this->filtreListBox->selectedItems();
	for(int i = 0; i < tags.size(); i++)
	{
		//Suppression dans le filtre
		((ToDoFrame*) this->parent())->filtre.push_back(tags.at(i)->text());
	}

	this->accept();
}

/**
 * Slot permettant le reset du filtre (deselection totale de tout les tags)
 */
void FiltreDialog::reset()
{
	//deselection
	this->filtreListBox->selectionModel()->clear();
}
