#include "tagseditordialog.h"
#include "todolist.h"
#include "todoframe.h"

/**
 * Constructeur
 */
TagsEditorDialog::TagsEditorDialog(QWidget *parent)
 : QDialog(parent)
{
		//contruction et initialisation
		this->setupUi();
		this->initUi();
		
		//event table
		QObject::connect(this->addTagButton, SIGNAL(clicked()), this, SLOT(addTag()));
		QObject::connect(this->okButton, SIGNAL(clicked()), this, SLOT(changed()));
		QObject::connect(this->cancelButton, SIGNAL(clicked()), this, SLOT(close()));
		QObject::connect(this->resetButton, SIGNAL(clicked()), this, SLOT(reset()));
}

/**
 * Methode permettant la construction de la boite de dialog
 */
void TagsEditorDialog::setupUi()
{
		//dialog options
		this->resize(350,300);
		this->setWindowTitle(QString::fromUtf8("Gestion des étiquettes..."));
		this->setModal(true);
		
		//creation des containers
		QVBoxLayout *v_main_layout = new QVBoxLayout();
		QHBoxLayout *h_section1_layout = new QHBoxLayout();
		QHBoxLayout *h_section2_layout = new QHBoxLayout();
		QGridLayout *grid_buttons_layout = new QGridLayout();
		this->addTagsGb = new QGroupBox(QString::fromUtf8("Ajouter une étiquette"));
		this->deleteTagsGb = new QGroupBox(QString::fromUtf8("Supprimer des étiquettes"));
		
		//creation des widgets
		this->addTagField = new QLineEdit();
		this->addTagField->setToolTip(QString::fromUtf8("Le nom du nouveau tag doit être unique"));
		this->addTagButton = new QPushButton("Ajouter");
		this->deleteListBox = new QListWidget();
		this->deleteListBox->setToolTip(QString::fromUtf8("Cliquez sur l'étiquette pour la supprimer"));
		this->deleteListBox->setSelectionMode(QAbstractItemView::MultiSelection);
		this->deleteListBox->setSortingEnabled(true);
		this->okButton = new QPushButton("Appliquer");
		this->resetButton = new QPushButton(QString::fromUtf8("Réinitialiser"));
		this->cancelButton = new QPushButton("Annuler");
		
		//alignement de la premiere section
		h_section1_layout->addWidget(this->addTagField);
		h_section1_layout->addWidget(this->addTagButton);
		this->addTagsGb->setLayout(h_section1_layout);
		
		//alignement de la seconde section
		h_section2_layout->addWidget(this->deleteListBox);
		this->deleteTagsGb->setLayout(h_section2_layout);
		
		//alignement des boutons
		grid_buttons_layout->addWidget(new QWidget(), 0,0,1,2);
		grid_buttons_layout->addWidget(this->resetButton, 0,2);
		grid_buttons_layout->addWidget(this->cancelButton, 0,3);
		grid_buttons_layout->addWidget(this->okButton, 0,4);
		
		//alignement global
		v_main_layout->addWidget(this->addTagsGb);
		v_main_layout->addWidget(this->deleteTagsGb);
		v_main_layout->addLayout(grid_buttons_layout);
		this->setLayout(v_main_layout);
}

/**
 * Methode permettant l'Initialisation/Remplissage des composants par defaut
 */
void TagsEditorDialog::initUi()
{
	
	//remplissage de la liste des tags
	std::vector<QString> tags = ToDoList::getInstance()->getTagsList();
	unsigned int i;
	for(i=0;i < tags.size(); i++)
	{
			QListWidgetItem * item = new QListWidgetItem(tags.at(i));
			this->deleteListBox->addItem(item);				
	}
	this->deleteListBox->sortItems();
}


/**
 * Slot permettant de réinitialiser la boite de dialogue
 */
 void TagsEditorDialog::reset()
 {
	//deselection
	this->deleteListBox->selectionModel()->clear();
	this->addTagField->setText("");
	
	//reset de la liste de tags à ajouter
	this->tagsToAdd.clear();
}
/**
 * Slot permettant d'ajouter un tag dans la liste d'attente
 */
void TagsEditorDialog::addTag()
{
	QString tagName = this->addTagField->text().toLower().trimmed();
	
		if(tagName == "")
		{
			QMessageBox::warning(this,"Attention",QString::fromUtf8("Une étiquette ne peut pas être vide !"));
			this->addTagField->setText("");

		}
		else if(this->tagsToAdd.contains(tagName) || ToDoList::getInstance()->containsTag(tagName))
		{
			QMessageBox::warning(this,"Attention",QString::fromUtf8("L'étiquette existe déjà !"));
		}
		else
		{
			QMessageBox::information(this,"Ajout OK",QString::fromUtf8("L'étiquette a été ajoutée !"));
			this->tagsToAdd.push_back(tagName);
			this->addTagField->setText("");
		}
}


/**
 * Slot permettant l'enregistrement des modifications (ajout et suppression) dans la liste de tags avant de quitter le dialog
 */
void TagsEditorDialog::changed()
{
	//Enregistrement des nouveaux tags
	for(int i=0; i < this->tagsToAdd.size(); i++)
	{
			ToDoList::getInstance()->addTag(this->tagsToAdd.at(i));
	}
	
	//Suppression des anciens tags
	QList<QListWidgetItem*> tags = this->deleteListBox->selectedItems();
	for(int i = 0; i < tags.size(); i++)
	{
		//Suppression dans la liste de tags (automatiquement reporter dans les taches)
		ToDoList::getInstance()->removeTag(tags.at(i)->text());
		
		//Suppression dans le filtre
		((ToDoFrame*) this->parent())->filtre.removeAll(tags.at(i)->text());
	}

	this->accept();
}


/**
 * Destructeur
 */
TagsEditorDialog::~TagsEditorDialog() {}
