#include <iostream>

#include "todoframe.h"
#include "tagseditordialog.h"
#include "createtaskdialog.h"
#include "edittaskdialog.h"
#include "filtredialog.h"
#include "task.h"
#include "todolist.h"

/**
 * Constructeur
 */
ToDoFrame::ToDoFrame(QWidget *parent)
 : QMainWindow(parent), notSaved(false)
{
	//creation de la fenetre
	this->setupUi();
	
	//event : menu de base
	QObject::connect(this->actionNew, SIGNAL(triggered()), this, SLOT(reset()));
	QObject::connect(this->actionSave, SIGNAL(triggered()), this, SLOT(save()));
	QObject::connect(this->actionQuit, SIGNAL(triggered()), this, SLOT(close()));
	QObject::connect(this->actionToolbar, SIGNAL(toggled(bool)), this, SLOT(showToolbar(bool)));
	QObject::connect(this->actionAbout, SIGNAL(triggered()), this, SLOT(about()));
	
	//event : gestion des tags
	QObject::connect(this->actionTags, SIGNAL(triggered()), this, SLOT(openTagsEditorDialog()));
	
	//event : creation d'un filtre par tags
	QObject::connect(this->actionFiltre, SIGNAL(triggered()), this, SLOT(openFiltreDialog()));
	
	
	//event : suppression d'une tache
	QObject::connect(this->actionDeleteTask, SIGNAL(triggered()), this, SLOT(deleteTask()));

	//event : ouverture de la boite de dialogue d'ajout d'une tache
	QObject::connect(this->actionAddTask, SIGNAL(triggered()), this, SLOT(openTaskCreateDialog()));

	//event : grisage de certains menu/actions 
	QObject::connect(this->currentTasksList, SIGNAL(itemSelectionChanged()), this, SLOT(updateQAction()));
	QObject::connect(this->finishedTasksList, SIGNAL(itemSelectionChanged()), this, SLOT(updateQAction()));
	QObject::connect(this->tabWidget, SIGNAL(currentChanged(int)), this, SLOT(updateQAction()));

	//event : ouverture de la boite de dialogue d'edition d'une tache
	QObject::connect(this->actionEditTask, SIGNAL(triggered()), this, SLOT(openTaskEditDialog()));
	QObject::connect(this->currentTasksList, SIGNAL(cellDoubleClicked(int,int)), this, SLOT(openTaskEditDialog()));
	QObject::connect(this->finishedTasksList, SIGNAL(cellDoubleClicked(int,int)), this, SLOT(openTaskEditDialog()));
}

/**
 * Methode permettant la conception de la fenetre principale
 */
void ToDoFrame::setupUi()
{
	//redimentionnement de la frame
	this->resize(800,500);
	this->setWindowTitle("ToDo Manager");
	this->setWindowIcon(QIcon("ressources/icon.png"));
	
	//creation de la barre de menu
	this->menuBar = new QMenuBar(this);

	//creation des menu
	this->menuFichier = new QMenu("&Fichier", this);
	this->menuEdition = new QMenu("&Edition", this);
	this->menuAide = new QMenu("&Aide", this);

	//creation des actions
	this->actionNew = new QAction("Nouvelle liste de tache...", this);
	this->actionNew->setShortcut(QKeySequence("Ctrl+N"));
	this->actionNew->setIcon(QIcon("ressources/new.png"));
	this->actionNew->setIconVisibleInMenu(false);
	this->actionSave = new QAction("Enregistrer", this);
	this->actionSave->setShortcut(QKeySequence("Ctrl+S"));
	this->actionSave->setIcon(QIcon("ressources/save.png"));
	this->actionSave->setIconVisibleInMenu(false);
	this->actionSave->setEnabled(false);
	this->actionQuit = new QAction("Quitter", this);
	this->actionQuit->setShortcut(QKeySequence("Ctrl+Q"));
	this->actionQuit->setIcon(QIcon("ressources/quit.png"));
	this->actionQuit->setIconVisibleInMenu(false);
	
	this->actionAddTask = new QAction(QString::fromUtf8("Ajouter une tâche..."), this);
	this->actionAddTask->setShortcut(QKeySequence("Ctrl+T"));
	this->actionAddTask->setIcon(QIcon("ressources/add.png"));
	this->actionAddTask->setIconVisibleInMenu(false);
	this->actionEditTask = new QAction(QString::fromUtf8("Éditer la tâche..."), this);
	this->actionEditTask->setIcon(QIcon("ressources/edit.png"));
	this->actionEditTask->setEnabled(false);
	this->actionDeleteTask = new QAction(QString::fromUtf8("Supprimer la tâche"), this);
	this->actionDeleteTask->setIcon(QIcon("ressources/delete.png"));
	this->actionDeleteTask->setEnabled(false);
	
	this->actionTags = new QAction(QString::fromUtf8("Gestion des étiquettes..."), this);
	this->actionTags->setShortcut(QKeySequence("Ctrl+E"));
	this->actionFiltre = new QAction("Filtrer...", this);
	this->actionFiltre->setShortcut(QKeySequence("Ctrl+F"));
	this->actionFiltre->setIcon(QIcon("ressources/filtre.png"));
	this->actionFiltre->setIconVisibleInMenu(false);
	this->actionToolbar = new QAction("Afficher la barre d'outils", this);
	this->actionToolbar->setCheckable(true);
	this->actionToolbar->setChecked(true);
	
	this->actionAbout = new QAction("A propos...", this);
	
	this->menuFichier->addAction(this->actionNew);
	this->menuFichier->addAction(this->actionSave);
	this->menuFichier->addSeparator();
	this->menuFichier->addAction(this->actionQuit);

	this->menuEdition->addAction(this->actionAddTask);
	this->menuEdition->addAction(this->actionTags);
	this->menuEdition->addAction(this->actionFiltre);
	this->menuEdition->addSeparator();
	this->menuEdition->addAction(this->actionToolbar);

	this->menuAide->addAction(this->actionAbout);

	//ajout des menus et de la menubar
	this->menuBar->addMenu(this->menuFichier);
	this->menuBar->addMenu(this->menuEdition);
	this->menuBar->addMenu(this->menuAide);
	this->setMenuBar(this->menuBar);

	//creation de la toolbar et ajout
	this->toolbar = new QToolBar("Toolbar principale");
	this->addToolBar(this->toolbar);
	this->toolbar->addAction(this->actionNew);
	this->toolbar->addAction(this->actionSave);
	this->toolbar->addSeparator();
	this->toolbar->addAction(this->actionAddTask);
	this->toolbar->addAction(this->actionEditTask);
	this->toolbar->addAction(this->actionDeleteTask);
	this->toolbar->addSeparator();
	this->toolbar->addAction(this->actionFiltre);
	this->toolbar->addSeparator();
	this->toolbar->addAction(this->actionQuit);

	//creation du tableau d'onglet
	this->tabWidget = new QTabWidget(this);
	
	this->currentTasksList = new QTableWidget(this->tabWidget);
	this->currentTasksList->setToolTip(QString::fromUtf8("Ce tableau contient les tâches en cours. Pour trier ces tâches, cliquer sur le nom de la colonne de votre choix !"));
	this->currentTasksList->setColumnCount(6);
	this->currentTasksList->setSelectionBehavior(QAbstractItemView::SelectRows);
	this->currentTasksList->setSelectionMode(QAbstractItemView::SingleSelection);

	//creation des items horizontals (nom des champs) pour l'onglet : taches en cours
	this->currentTasksList->setHorizontalHeaderItem(0,new QTableWidgetItem("Nom"));
	this->currentTasksList->setHorizontalHeaderItem(1,new QTableWidgetItem("Description"));
	this->currentTasksList->setHorizontalHeaderItem(2,new QTableWidgetItem(QString::fromUtf8("Priorité")));
	this->currentTasksList->setHorizontalHeaderItem(3,new QTableWidgetItem(QString::fromUtf8("Échéance")));
	this->currentTasksList->setHorizontalHeaderItem(4,new QTableWidgetItem("Ajout"));
	this->currentTasksList->setHorizontalHeaderItem(5,new QTableWidgetItem(QString::fromUtf8("Étiquettes")));

	this->currentTasksList->setSortingEnabled(true);
	this->currentTasksList->horizontalHeader()->setCascadingSectionResizes(true);
	this->currentTasksList->horizontalHeader()->setDefaultSectionSize(80);
	this->currentTasksList->horizontalHeader()->resizeSection(0,150);
	this->currentTasksList->horizontalHeader()->resizeSection(1,250);
	this->currentTasksList->verticalHeader()->setDefaultSectionSize(50);
	this->currentTasksList->horizontalHeader()->setProperty("showSortIndicator", QVariant(true));
	this->currentTasksList->horizontalHeader()->setStretchLastSection(true);
	
	//creation des items horizontals (nom des champs) pour l'onglet : taches terminees
	this->finishedTasksList = new QTableWidget(this->tabWidget);
	this->finishedTasksList->setToolTip(QString::fromUtf8("Ce tableau contient les tâches terminées. Pour trier ces tâches, cliquer sur le nom de la colonne de votre choix !"));
	this->finishedTasksList->setColumnCount(6);
	this->finishedTasksList->setSelectionBehavior(QAbstractItemView::SelectRows);
	this->finishedTasksList->setSelectionMode(QAbstractItemView::SingleSelection);

	this->finishedTasksList->setHorizontalHeaderItem(0,new QTableWidgetItem("Nom"));
	this->finishedTasksList->setHorizontalHeaderItem(1,new QTableWidgetItem("Description"));
	this->finishedTasksList->setHorizontalHeaderItem(2,new QTableWidgetItem(QString::fromUtf8("Priorité")));
	this->finishedTasksList->setHorizontalHeaderItem(3,new QTableWidgetItem(QString::fromUtf8("Échéance")));
	this->finishedTasksList->setHorizontalHeaderItem(4,new QTableWidgetItem("Ajout"));
	this->finishedTasksList->setHorizontalHeaderItem(5,new QTableWidgetItem(QString::fromUtf8("Étiquettes")));

	this->finishedTasksList->setSortingEnabled(true);
	this->finishedTasksList->horizontalHeader()->setCascadingSectionResizes(true);
	this->finishedTasksList->horizontalHeader()->setDefaultSectionSize(80);
	this->finishedTasksList->horizontalHeader()->resizeSection(0,150);
	this->finishedTasksList->horizontalHeader()->resizeSection(1,250);
	this->finishedTasksList->verticalHeader()->setDefaultSectionSize(50);

	this->finishedTasksList->horizontalHeader()->setProperty("showSortIndicator", QVariant(true));
	this->finishedTasksList->horizontalHeader()->setStretchLastSection(true);

	//ajout des deux onglets
	this->tabWidget->addTab(this->currentTasksList, QString::fromUtf8("Tâches en cours"));
	this->tabWidget->addTab(this->finishedTasksList, QString::fromUtf8("Tâches terminées"));
	this->setCentralWidget(this->tabWidget);

	//remplissage des onglets
	this->repaintTabs();
}

/**
 * Slot permettant de griser ou de dégriser certains elements du menu
 * - menu "Enregistrer"
 * - menu "Modifier une tache" et "Supprimer une tache"
 */
void ToDoFrame::updateQAction()
{
	//activation - desactivation du bouton enregistrer
	if(this->notSaved)
		this->actionSave->setEnabled(true);
	else
		this->actionSave->setEnabled(false);
	
	//activation desactivation du bouton d'edition et de suppression d'une tache	
	if(this->tabWidget->currentIndex() == 0) //onglet1

	{
		if(this->currentTasksList->selectedItems().size() > 0)
		{
				this->actionEditTask->setEnabled(true);
				this->actionDeleteTask->setEnabled(true);
		}
		else
		{
				this->actionEditTask->setEnabled(false);
				this->actionDeleteTask->setEnabled(false);
		}
	}
	else //onglet2
	{
		if(this->finishedTasksList->selectedItems().size() > 0)
		{
				this->actionEditTask->setEnabled(true);
				this->actionDeleteTask->setEnabled(true);
		}
		else
		{
				this->actionEditTask->setEnabled(false);
				this->actionDeleteTask->setEnabled(false);
		}
	}

		
}

/**
 * Methode permettant de mettre à jours le contenu des deux listes de taches
 * (applique egalement le filtrage)
 */
void ToDoFrame::repaintTabs()
{
	//on trouve le nombre de ligne pour chacun des deux onglets (avec filtrage ou non)
	int nbCurrent = 0;
	int nbOld = 0;
	if(this->filtre.size() > 0)
	{
		//pour chaque tache, on cherche si elle contient le filtre
		std::vector<Task*> tasks = ToDoList::getInstance()->getTasks();
		unsigned int index;
		for(index = 0; index < tasks.size(); index++)
		{
			for(int i = 0; i < this->filtre.size(); i++)
			{
					//contient le filtre
					if(tasks.at(index)->containsTag(this->filtre.at(i)))
					{
						//incrementation du compteur selon son statut
						if(tasks.at(index)->isFinished())
							nbOld++;
						else
							nbCurrent++;
							
						break;
					}
			}
		}
	}
	else
	{
		nbCurrent = ToDoList::getInstance()->countCurrentTasks();
		nbOld = ToDoList::getInstance()->countFinishedTasks();
	}
	
	
	//mise à jours des nom d'onglet
	QString numActive;
	numActive.setNum(nbCurrent);
	this->tabWidget->setTabText(0,QString::fromUtf8("Tâches en cours (")+numActive+")");
	
	QString numFinished;
	numFinished.setNum(nbOld);
	this->tabWidget->setTabText(1,QString::fromUtf8("Tâches terminées (")+numFinished+")");
	
	//mise en place des taches en cours
	this->currentTasksList->setRowCount(nbCurrent);
	std::vector<Task*> activeTasks = ToDoList::getInstance()->getCurrentTasks();
	unsigned int index;
	int realIndex = 0;
	for(index = 0; index < activeTasks.size(); index++)
	{
		
		//systeme de filtrage
		bool inFiltre = false;
		for(int i = 0; i < this->filtre.size(); i++)
		{
				if(activeTasks.at(index)->containsTag(this->filtre.at(i)))
				{
					inFiltre = true;
					break;
				}
		}
		
		if(inFiltre || this->filtre.size() == 0)
		{
		
		Task* t = activeTasks.at(index);
		this->currentTasksList->setItem(realIndex, 0, new QTableWidgetItem(t->getName()));
		this->currentTasksList->setItem(realIndex, 1, new QTableWidgetItem(t->getDescription()));
		this->currentTasksList->setItem(realIndex, 2, new QTableWidgetItem(t->getPriorityString()));
		this->currentTasksList->setItem(realIndex, 3, new QTableWidgetItem(t->getDeadLineString()));
		this->currentTasksList->setItem(realIndex, 4, new QTableWidgetItem(t->getCreateDateString()));
		this->currentTasksList->setItem(realIndex, 5, new QTableWidgetItem(t->getTagsString()));
		
		//items seulement selectable (pas editable)
		this->currentTasksList->item(realIndex, 0)->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		this->currentTasksList->item(realIndex, 1)->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		this->currentTasksList->item(realIndex, 2)->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		this->currentTasksList->item(realIndex, 3)->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		this->currentTasksList->item(realIndex, 4)->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		this->currentTasksList->item(realIndex, 5)->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		
		realIndex++;
		}
	}
	
	//mise en place des taches terminées
	this->finishedTasksList->setRowCount(nbOld);
	std::vector<Task*> finishedTasks = ToDoList::getInstance()->getFinishedTasks();
	realIndex = 0;
	for(index = 0; index < finishedTasks.size(); index++)
	{
		
		//systeme de filtrage
		bool inFiltre = false;
		for(int i = 0; i < this->filtre.size(); i++)
		{
				if(finishedTasks.at(index)->containsTag(this->filtre.at(i)))
				{
					inFiltre = true;
					break;
				}
		}
		
		if(inFiltre || this->filtre.size() == 0)
		{
			
		Task* t = finishedTasks.at(index);
		this->finishedTasksList->setItem(realIndex, 0, new QTableWidgetItem(t->getName()));
		this->finishedTasksList->setItem(realIndex, 1, new QTableWidgetItem(t->getDescription()));
		this->finishedTasksList->setItem(realIndex, 2, new QTableWidgetItem(t->getPriorityString()));
		this->finishedTasksList->setItem(realIndex, 3, new QTableWidgetItem(t->getDeadLineString()));
		this->finishedTasksList->setItem(realIndex, 4, new QTableWidgetItem(t->getCreateDateString()));
		this->finishedTasksList->setItem(realIndex, 5, new QTableWidgetItem(t->getTagsString()));
		
		//items seulement selectable (pas editable)
		this->finishedTasksList->item(realIndex, 0)->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		this->finishedTasksList->item(realIndex, 1)->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		this->finishedTasksList->item(realIndex, 2)->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		this->finishedTasksList->item(realIndex, 3)->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		this->finishedTasksList->item(realIndex, 4)->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		this->finishedTasksList->item(realIndex, 5)->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		
		realIndex++;
		}
	}
	
}

/**
 * Slot permettant l'ouverture d'une dialogue d'édition d'une tache
 */
void ToDoFrame::openTaskEditDialog()
{
	
	//on recupere le nom de la tache a editer
	QString taskNameId;
	if(this->tabWidget->currentIndex() == 0) //onglet1
	{
		taskNameId = this->currentTasksList->selectedItems().at(0)->text();
	}
	else //onglet2
	{
		taskNameId = this->finishedTasksList->selectedItems().at(0)->text();
	}
	
	//affichage du dialog
	EditTaskDialog edit(this,taskNameId);
	edit.exec();
	
	//repaint
	if(edit.result() == 1)
	{
		std::cout << ">> EDITION D'UNE TACHE..." << std::endl;
		
		this->repaintTabs();
		this->notSaved = true;
		this->updateQAction();
	}
}

/**
 * Slot permettant l'ouverture d'une dialogue permettant la creation d'une nouvelle tache
 */
void ToDoFrame::openTaskCreateDialog()
{	
	//affichage de la boite de dialogue de creation
	CreateTaskDialog create(this);
	create.exec();
	
	//repaint
	if(create.result() == 1)
	{
		std::cout << ">> NOUVELLE TACHE..." << std::endl;
		
		this->repaintTabs();
		this->notSaved = true;
		this->updateQAction();
	}
}

/**
 * Slot permettant l'ouverture d'une dialogue permettant la creation un filtre par mot-clé
 */
void ToDoFrame::openFiltreDialog()
{	
	//affichage de la boite de dialogue de creation
	FiltreDialog filtre(this);
	filtre.exec();
	
	//repaint
	if(filtre.result() == 1)
	{
		std::cout << ">> MODIFICATION DU FILTRE..." << std::endl;
		
		this->repaintTabs();
	}
}

/**
 * Slot permettant le grisage ou non de la tool-barre
 * @param checked status de la tool-barrre
 */
void ToDoFrame::showToolbar(bool checked)
{
	if(!checked)
		this->toolbar->setVisible(false);
	else
		this->toolbar->setVisible(true);
}

/**
 * Slot permettant d'effectuer une remise à zero des taches. 
 * (lancement d'une boite de confirmation)
 */
void ToDoFrame::reset()
{
	if(ToDoList::getInstance()->countTasks() > 0)
	{
		int ret = QMessageBox::warning(
			this,
			"Attention",
			QString::fromUtf8("Votre action supprimera toutes vos tâches existantes !\nVoulez-vous continuer ?"), 		
			QMessageBox::No | QMessageBox::Yes,
			QMessageBox::No);

		if(ret == QMessageBox::Yes)
		{
			std::cout << ">> NOUVELLE LISTE..." << std::endl;
			ToDoList::getInstance()->getTasks().clear();
			this->repaintTabs();
			this->save();
			
			this->notSaved = false;
			this->updateQAction();
		}
	}
}

/**
 * Slot permettant l'ouverture de la dialog de gestion des tags
 */
void ToDoFrame::openTagsEditorDialog()
{
		//creation du dialog
		TagsEditorDialog te (this);
		te.exec();
	
		//repaint
		if(te.result() == 1)
		{
			std::cout << ">> MODIFICATION DES ETIQUETTES..." << std::endl;
			
			this->repaintTabs();
			this->notSaved = true;
			this->updateQAction();
		}
}

/**
 * Slot permettant de sauvegarder l'application
 */
void ToDoFrame::save()
{
	std::cout << ">> ENREGISTREMENT DANS LE FICHIER XML..." << std::endl;
	ToDoList::getInstance()->saveXML();
	
	this->notSaved = false;
	this->updateQAction();
}

/**
 * Slot permettant de supprimer une tache 
 */
void ToDoFrame::deleteTask()
{
	bool doRepaint = false;
	
	// suppression de la tache
	if(this->tabWidget->currentIndex() == 0) //onglet1
	{
		doRepaint = ToDoList::getInstance()->removeTask(this->currentTasksList->selectedItems().at(0)->text());
	}
	else //onglet2
	{
		doRepaint = ToDoList::getInstance()->removeTask(this->finishedTasksList->selectedItems().at(0)->text());
	}
	
	//repaint
	if(doRepaint)
	{
		this->repaintTabs();
		this->notSaved = true;
		this->updateQAction();
		std::cout << ">> SUPPRESSION D'UNE TACHE..." << std::endl;
	}
}

/**
 * Redefinition du slot permettant de quitter l'application
 * (lancement d'une boite de dialogue de confirmation, si pas de sauvegarde) 
 */
void ToDoFrame::closeEvent(QCloseEvent *event)
{
	bool doQuit = true;
	
	if(this->notSaved)
		doQuit = this->quitDialog();

	if(doQuit)
	{
		event->accept();
		std::cout << ">> FERMETURE..." << std::endl;
	}
	else
	{
		event->ignore();
	}
}

/**
 * Methode permettant de demander à l'utilisateur une confirmation d'interruption du programme
 */
bool ToDoFrame::quitDialog()
{
	bool doQuit = true;
	
	//demande à l'utilisateur
	int ret = QMessageBox::question(
			this,
			"Quitter",
            QString::fromUtf8("La liste de tâche a été modifiée.\nVoulez-vous enregistrer les changements ?"), 
			QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel,
            QMessageBox::Save
            );
	
	 //verification du code retour
	 if(ret == QMessageBox::Save)
	 {
			this->save();
	 }
	 else if(ret == QMessageBox::Cancel)
	 {
			doQuit = false;
	 }
	 
	return doQuit; 
}

/**
 * Slot permettant l'affichage des informations de l'application (about())
 */
void ToDoFrame::about()
{
		QMessageBox::about(
		this, 
		"A propos...",
		QString::fromUtf8("<span style='font-size:25px;'>ToDo Manager</span><br/>version 1.0<br/><strong>par Haehnel Jonathan</strong><br/><br/><em>ToDo Manager</em> permet de créer et de gèrer sa liste de tâches à réaliser. Cette application a été réalisé dans le cadre d'un projet en 3ème année de Licence Informatique."));
}

/**
 * Destructeur
 */
ToDoFrame::~ToDoFrame()
{
}
